require "import"
import "android.widget.*"
import "android.view.*"
import "android.media.MediaPlayer"
import "android.media.AudioManager"
import "android.content.*"
import "android.app.AlertDialog"
import "android.net.Uri"
import "android.provider.DocumentsContract"
import "android.os.Environment"
import "android.os.Handler"
import "java.io.File"
import "android.os.Build"

-- 全局变量
local mediaPlayer = MediaPlayer()
local currentIndex = 1
local musicList = {}
local currentMusicUri = ""
local SEEK_TIME = 10000 -- 快进/快退时间(毫秒)
local currentPitch = 1.0 -- 当前变调值(1.0=原调)
local currentSpeed = 1.0 -- 当前播放速度(1.0=原速)
local isSeekable = false -- 是否可跳转进度

-- 获取SharedPreferences
local sharedPref = activity.getSharedPreferences("MusicPlayerPrefs", Context.MODE_PRIVATE)

-- 24个半音变调选项 (从-12到+12半音)
local semitoneRatios = {}
local pitchNames = {}
for i = -12, 12 do
    local ratio = 2^(i/12)
    table.insert(semitoneRatios, ratio)
    if i == 0 then
        table.insert(pitchNames, "原调")
    elseif i < 0 then
        table.insert(pitchNames, string.format("降%d半音", math.abs(i)))
    else
        table.insert(pitchNames, string.format("升%d半音", i))
    end
end

-- 变速选项 (0.5到3倍，步长0.25)
local speedValues = {0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0, 2.25, 2.5, 2.75, 3.0}
local speedNames = {
    "0.5x慢速", "0.75x慢速", "1.0x原速", 
    "1.25x快速", "1.5x快速", "1.75x快速",
    "2.0x快速", "2.25x快速", "2.5x快速",
    "2.75x快速", "3.0x快速"
}

-- 主布局
layout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "fill",
    layout_height = "fill",
    padding = "16dp",
    {
        LinearLayout,
        orientation = "horizontal",
        layout_width = "fill",
        layout_marginBottom = "16dp",
        {
            Button,
            id = "btnSelectDir",
            text = "选择音乐目录",
            layout_width = "0dp",
            layout_weight = "1",
            layout_marginRight = "8dp"
        },
        {
            Button,
            id = "btnPitch",
            text = "变调: 原调",
            layout_width = "0dp",
            layout_weight = "1",
            layout_marginRight = "8dp"
        },
        {
            Button,
            id = "btnSpeed",
            text = "速度: 1.0x",
            layout_width = "0dp",
            layout_weight = "1"
        }
    },
    {
        TextView,
        id = "txtCurrentMusic",
        text = "未选择音乐",
        textSize = "18sp",
        layout_width = "fill",
        gravity = "center",
        padding = "16dp"
    },
    {
        SeekBar,
        id = "seekBar",
        layout_width = "fill",
        layout_marginTop = "16dp",
        layout_marginBottom = "16dp"
    },
    {
        ListView,
        id = "listView",
        layout_width = "fill",
        layout_height = "0dp",
        layout_weight = "1",
        dividerHeight = "1dp"
    },
    -- 底部控制栏
    {
        LinearLayout,
        orientation = "horizontal",
        layout_width = "fill",
        gravity = "center",
        layout_marginTop = "16dp",
        {
            Button,
            id = "btnPrev",
            text = "上一首",
            layout_width = "80dp",
            layout_height = "60dp",
            layout_margin = "4dp",
            textSize = "12sp"
        },
        {
            Button,
            id = "btnRewind",
            text = "快退10秒",
            layout_width = "80dp",
            layout_height = "60dp",
            layout_margin = "4dp",
            textSize = "12sp"
        },
        {
            Button,
            id = "btnPlayPause",
            text = "播放",
            layout_width = "100dp",
            layout_height = "80dp",
            layout_margin = "4dp",
            textSize = "14sp"
        },
        {
            Button,
            id = "btnForward",
            text = "快进10秒",
            layout_width = "80dp",
            layout_height = "60dp",
            layout_margin = "4dp",
            textSize = "12sp"
        },
        {
            Button,
            id = "btnNext",
            text = "下一首",
            layout_width = "80dp",
            layout_height = "60dp",
            layout_margin = "4dp",
            textSize = "12sp"
        }
    }
}

-- 加载布局
activity.setContentView(loadlayout(layout))

-- 适配器
adapter = LuaAdapter(activity, musicList, {
    TextView,
    id = "text",
    textSize = "16sp",
    padding = "12dp",
    layout_width = "fill"
})

listView.setAdapter(adapter)

-- 保存音乐列表到SharedPreferences
local function saveMusicList()
    local editor = sharedPref.edit()
    
    -- 保存音乐列表
    editor.putInt("musicListSize", #musicList)
    for i, music in ipairs(musicList) do
        editor.putString("music_"..i.."_text", music.text)
        editor.putString("music_"..i.."_uri", music.uri)
    end
    
    editor.apply()
end

-- 从SharedPreferences加载音乐列表
local function loadMusicList()
    -- 加载音乐列表
    musicList = {}
    local size = sharedPref.getInt("musicListSize", 0)
    for i = 1, size do
        local uri = sharedPref.getString("music_"..i.."_uri", "")
        if uri ~= "" then
            table.insert(musicList, {
                text = sharedPref.getString("music_"..i.."_text", ""),
                uri = uri
            })
        end
    end
    
    if #musicList > 0 then
        -- 更新适配器
        adapter.clear()
        for k,v in pairs(musicList) do
            adapter.add(v)
        end
        adapter.notifyDataSetChanged()
        return true
    end
    return false
end

-- 检查URI是否可访问
local function isUriAccessible(uriString)
    local success, uri = pcall(function() return Uri.parse(uriString) end)
    if not success then return false end
    
    local contentResolver = activity.getContentResolver()
    local cursor = contentResolver.query(uri, nil, nil, nil, nil)
    if cursor ~= nil then
        cursor.close()
        return true
    end
    return false
end

-- 添加单个音乐文件到列表
local function addMusicFile(uri, name)
    -- 检查是否已存在
    for _, music in ipairs(musicList) do
        if music.uri == uri.toString() then
            return false
        end
    end
    
    -- 检查URI是否可访问
    if not isUriAccessible(uri.toString()) then
        return false
    end
    
    table.insert(musicList, {
        text = name,
        uri = uri.toString()
    })
    
    -- 更新适配器
    adapter.add{
        text = name,
        uri = uri.toString()
    }
    adapter.notifyDataSetChanged()
    
    -- 保存更新后的列表
    saveMusicList()
    return true
end

-- 更新播放状态显示
local function updatePlayStatus()
    if currentMusicUri ~= "" then
        for i, v in ipairs(musicList) do
            if v.uri == currentMusicUri then
                txtCurrentMusic.text = v.text
                -- 更新变调显示
                local closestPitchIndex = 13 -- 默认为原调
                local minPitchDiff = math.abs(currentPitch - semitoneRatios[13])
                for j, pitch in ipairs(semitoneRatios) do
                    local diff = math.abs(currentPitch - pitch)
                    if diff < minPitchDiff then
                        minPitchDiff = diff
                        closestPitchIndex = j
                    end
                end
                btnPitch.text = "变调: "..pitchNames[closestPitchIndex]
                
                -- 更新速度显示
                local closestSpeedIndex = 3 -- 默认为1.0x
                local minSpeedDiff = math.abs(currentSpeed - speedValues[3])
                for k, speed in ipairs(speedValues) do
                    local diff = math.abs(currentSpeed - speed)
                    if diff < minSpeedDiff then
                        minSpeedDiff = diff
                        closestSpeedIndex = k
                    end
                end
                btnSpeed.text = "速度: "..speedNames[closestSpeedIndex]
                break
            end
        end
    else
        txtCurrentMusic.text = "未选择音乐"
        btnPitch.text = "变调: 原调"
        btnSpeed.text = "速度: 1.0x"
    end
end

-- 设置播放参数（变调和速度）
local function setPlaybackParams(pitch, speed)
    if Build.VERSION.SDK_INT >= 23 then
        local success, ret = pcall(function()
            local params = mediaPlayer.getPlaybackParams()
            params.setPitch(pitch)
            params.setSpeed(speed)
            mediaPlayer.setPlaybackParams(params)
            return true
        end)
        
        if success then
            currentPitch = pitch
            currentSpeed = speed
            updatePlayStatus()
        else
            Toast.makeText(activity, "参数设置失败", Toast.LENGTH_SHORT).show()
        end
    else
        Toast.makeText(activity, "您的设备不支持此功能", Toast.LENGTH_SHORT).show()
    end
end

-- 显示变调选择对话框
local function showPitchDialog()
    local dlg = AlertDialog.Builder(activity)
    dlg.setTitle("选择变调")
    
    dlg.setItems(pitchNames, {onClick=function(dialog, which)
        setPlaybackParams(semitoneRatios[which+1], currentSpeed)
    end})
    
    dlg.setNegativeButton("取消", nil)
    dlg.show()
end

-- 显示变速选择对话框
local function showSpeedDialog()
    local dlg = AlertDialog.Builder(activity)
    dlg.setTitle("选择播放速度")
    
    dlg.setItems(speedNames, {onClick=function(dialog, which)
        setPlaybackParams(currentPitch, speedValues[which+1])
    end})
    
    dlg.setNegativeButton("取消", nil)
    dlg.show()
end

-- 快进功能
local function fastForward()
    if isSeekable then
        local newPosition = mediaPlayer.getCurrentPosition() + SEEK_TIME
        if newPosition > mediaPlayer.getDuration() then
            newPosition = mediaPlayer.getDuration()
        end
        mediaPlayer.seekTo(newPosition)
        seekBar.progress = newPosition
    end
end

-- 快退功能
local function rewind()
    if isSeekable then
        local newPosition = mediaPlayer.getCurrentPosition() - SEEK_TIME
        if newPosition < 0 then
            newPosition = 0
        end
        mediaPlayer.seekTo(newPosition)
        seekBar.progress = newPosition
    end
end

-- 播放音乐函数
local function playMusic(uri)
    -- 检查URI是否可访问
    if not isUriAccessible(uri) then
        Toast.makeText(activity, "无法访问音乐文件，请重新选择", Toast.LENGTH_SHORT).show()
        return
    end
    
    local success, err = pcall(function()
        mediaPlayer.reset()
        mediaPlayer.setDataSource(activity, Uri.parse(uri))
        mediaPlayer.prepareAsync()
        currentMusicUri = uri
        isSeekable = false
        btnPlayPause.text = "加载中..."
        
        for i, v in ipairs(musicList) do
            if v.uri == uri then
                currentIndex = i
                break
            end
        end
    end)
    
    if not success then
        Toast.makeText(activity, "播放错误: "..tostring(err), Toast.LENGTH_SHORT).show()
    end
end

-- 扫描音乐文件
local function scanMusicFilesFromUri(treeUri)
    local contentResolver = activity.getContentResolver()
    local docId = DocumentsContract.getTreeDocumentId(treeUri)
    local childrenUri = DocumentsContract.buildChildDocumentsUriUsingTree(treeUri, docId)
    
    local cursor = contentResolver.query(childrenUri, 
        {DocumentsContract.Document.COLUMN_DISPLAY_NAME, DocumentsContract.Document.COLUMN_DOCUMENT_ID}, 
        nil, nil, nil)
    
    if cursor ~= nil then
        while cursor.moveToNext() do
            local name = cursor.getString(0)
            local docId = cursor.getString(1)
            local fileUri = DocumentsContract.buildDocumentUriUsingTree(treeUri, docId)
            
            if name:lower():match("%.mp3$") or name:lower():match("%.wav$") or name:lower():match("%.ogg$") then
                -- 尝试获取文件权限
                pcall(function()
                    activity.getContentResolver().takePersistableUriPermission(
                        fileUri,
                        Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION
                    )
                end)
                
                addMusicFile(fileUri, name)
            end
        end
        cursor.close()
    end
end

-- 播放下一首
local function playNext()
    if #musicList == 0 then return end
    
    currentIndex = currentIndex + 1
    if currentIndex > #musicList then
        currentIndex = 1
    end
    
    -- 检查下一首是否可访问
    local nextUri = musicList[currentIndex].uri
    if not isUriAccessible(nextUri) then
        Toast.makeText(activity, "无法访问下一首音乐，已跳过", Toast.LENGTH_SHORT).show()
        playNext() -- 递归尝试下一首
        return
    end
    
    playMusic(nextUri)
end

-- 播放上一首
local function playPrev()
    if #musicList == 0 then return end
    
    currentIndex = currentIndex - 1
    if currentIndex < 1 then
        currentIndex = #musicList
    end
    
    -- 检查上一首是否可访问
    local prevUri = musicList[currentIndex].uri
    if not isUriAccessible(prevUri) then
        Toast.makeText(activity, "无法访问上一首音乐，已跳过", Toast.LENGTH_SHORT).show()
        playPrev() -- 递归尝试上一首
        return
    end
    
    playMusic(prevUri)
end

-- 初始化MediaPlayer
local function initMediaPlayer()
    mediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC)
    
    mediaPlayer.setOnPreparedListener({
        onPrepared = function(mp)
            isSeekable = true
            seekBar.max = mp.getDuration()
            updatePlayStatus()
            mp.start()
            btnPlayPause.text = "暂停"
            
            if Build.VERSION.SDK_INT >= 23 then
                setPlaybackParams(currentPitch, currentSpeed)
            end
        end
    })
    
    mediaPlayer.setOnCompletionListener({
        onCompletion = function(mp)
            playNext()
        end
    })
    
    mediaPlayer.setOnErrorListener({
        onError = function(mp, what, extra)
            Toast.makeText(activity, "播放错误: "..what, Toast.LENGTH_SHORT).show()
            return true
        end
    })
end

-- 按钮事件
btnPlayPause.onClick = function()
    if mediaPlayer.isPlaying() then
        mediaPlayer.pause()
        btnPlayPause.text = "播放"
    else
        if mediaPlayer.getCurrentPosition() > 0 then
            mediaPlayer.start()
            btnPlayPause.text = "暂停"
        elseif #musicList > 0 then
            currentIndex = 1
            playMusic(musicList[1].uri)
        end
    end
end

btnNext.onClick = function()
    playNext()
end

btnPrev.onClick = function()
    playPrev()
end

btnForward.onClick = function()
    fastForward()
end

btnRewind.onClick = function()
    rewind()
end

btnPitch.onClick = function()
    showPitchDialog()
end

btnSpeed.onClick = function()
    showSpeedDialog()
end

btnSelectDir.onClick = function()
    local intent = Intent(Intent.ACTION_OPEN_DOCUMENT_TREE)
    activity.startActivityForResult(intent, 1)
end

-- 列表点击事件
listView.onItemClick = function(parent, view, position, id)
    currentIndex = position + 1
    playMusic(musicList[currentIndex].uri)
end

-- 进度条事件
seekBar.setOnSeekBarChangeListener{
    onProgressChanged = function(seekBar, progress, fromUser)
        if fromUser and isSeekable then
            mediaPlayer.seekTo(progress)
        end
    end
}

-- 更新进度条
local handler = Handler()
local updateSeekBar = function()
    if isSeekable and mediaPlayer.isPlaying() then
        seekBar.progress = mediaPlayer.getCurrentPosition()
    end
    handler.postDelayed(updateSeekBar, 1000)
end
handler.postDelayed(updateSeekBar, 1000)

-- 处理目录选择结果
function onActivityResult(requestCode, resultCode, intent)
    if requestCode == 1 and resultCode == activity.RESULT_OK then
        local uri = intent.getData()
        -- 获取目录权限
        pcall(function()
            activity.getContentResolver().takePersistableUriPermission(
                uri, 
                Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION
            )
        end)
        scanMusicFilesFromUri(uri)
    end
end

-- 退出时释放资源
function onDestroy()
    if mediaPlayer ~= nil then
        mediaPlayer.release()
    end
end

-- 初始化
initMediaPlayer()

-- 尝试加载上次的音乐列表
if not loadMusicList() then
    -- 如果没有保存的音乐列表，显示默认状态
    updatePlayStatus()
end